import datetime
import pandas as pd
from collections import namedtuple

BroadcastDetails = namedtuple('BroadcastDetails', 'year,quarter,month,week')

def get_broadcast_week_start(date):
    """ returns the broadcast week start for a given date
    """
    if type(date) is pd.Timestamp:
        date = date.to_pydatetime().date()
    elif type(date) is datetime.datetime:
        date = date.date()

    if date.weekday() != 0:
        date = date - datetime.timedelta(days=(date.weekday()))
    return date


def get_broadcast_week_start_from_year_and_week(broadcast_year,
                                                broadcast_week):
    """ returns the start of broadcast week for a given broadcast
    year and broadcast week
    """
    if get_broadcast_weeks_in_year(broadcast_year) < broadcast_week:
        return None

    first_day_of_broadcast_year = _get_first_day_of_broadcast_year(
        broadcast_year)
    return first_day_of_broadcast_year + datetime.timedelta(days=(7 * (
        broadcast_week - 1)))


def get_broadcast_details(date):
    """ returns a named tuple of BroadcastDetails
    for a given date
    """
    if type(date) is pd.Timestamp:
        date = date.to_pydatetime().date()
    elif type(date) is datetime.datetime:
        date = date.date()

    if date.weekday() != 6:
        date = date + datetime.timedelta(days=(6 - date.weekday()))

    first_day_of_broadcast_year = _get_first_day_of_broadcast_year(date.year)
    week = int((date - first_day_of_broadcast_year).days / 7) + 1
    quarter = (date.month - 1) // 3 + 1

    return BroadcastDetails(year=date.year, quarter=quarter,
                            month=date.month, week=week)


def get_broadcast_weeks_in_year(year):
    """ returns the number of broadcast weeks in a given year
    """
    first_day = datetime.date(year, 1, 1)

    # if the first day is a Sunday
    if first_day.weekday() == 6:
        return 53
    # if it is a leap year, and the first day is a Saturday
    elif _is_leap_year(year) and first_day.weekday() == 5:
        return 53
    return 52




def _get_first_day_of_broadcast_year(year):
    """ returns the first day of the broadcast year for a given year
    """
    first_day = datetime.date(year, 1, 1)

    # if the week doesn't start on Monday, get the previous Monday
    if first_day.weekday() != 0:
        first_day = first_day - datetime.timedelta(days=(first_day.weekday()))

    return first_day


def _is_leap_year(year):
    """ returns true if the given year is a leap year
    """
    return year % 4 == 0 and (year % 400 == 0 or year % 100 != 0)
